#### # # #  #  #   #   #    #     #      #      #     #    #   #  #  # # # ###
# 
# Kevin McMahon May 22, 2003
# Modified by Saurabh Abhyankar - Dec 10, 2003
# 
# Contact:   saurabh.abhyankar@crystaldecisions.com
#
# Purpose:   Change table of content headings in a xml file so that 
# 		headings below sections are indented an appropriate amount.
#
# Pre-condition: xml file generated by the Webworks WebSphere template. 
#		NOTE: For each paragraph tag in the FrameMaker TOC file, the WebSphere 
#		template writes only the opening tag. This Perl script writes the 
#		closing tags so the table of contents (xml file) will have its levels
#		indented by the correct amount.
#
# Post-condition: The xml file with the table of contents adjusted.
#  
# Syntax:    perl c:\path\adjustWebSphereTOC.pl c:\path\xmlfile.xml
#
### # # #  #  #   #   #    #     #      #      #     #    #   #  #  # # # ###


### # # #  #  #   #   #    #     #      #      #     #    #   #  #  # # # ###
#
# Main body of script
#

use File::Copy;
use Win32::File;
use strict;

## # #  #   #    SUBROUTINES    #   #  # # ## 


sub process_file;
sub make_writable;

## # #  #   #    GLOBAL VARIABLES    #   #  # # ## 

(my $section_increment) = 0;
(my $previous_level) = 0;
(my $current_level) = 0;
(my $open_node_count) = 0;
(my $level_difference) = 0;
(my $in_node_section) = 0;
(my $in_sec1) = 0;
(my $in_sec2) = 0;
(my $heading_error) = 0;
(my $output_line_num) = 0;

## # #  #   #    PROCESS COMMAND LINE ARGS   #   #  # # ## 

(my $file_to_process) = $ARGV[0];

process_file;

### # # #  #  #   #   #    #     #      #      #     #    #   #  #  # # # ###
# 
# sub process_file 
#
# Parameters: None.
#

sub process_file 
{
	# Make sure current file is writable	
	&make_writable($file_to_process);			

	
	# Create backup file				
	copy($file_to_process, "$file_to_process.orig") || die "Can't create backup: $!";		

	# Open backup file for reading
	open(ORIGINAL, "<$file_to_process.orig") || die "Can't open backup: $!";	

	# Open original filename for writing
	open(COPY, ">$file_to_process") || die "Can't open for writing: $!";		

	# Open logfile for writing
	open(LOGFILE, ">adjustHXSTOC.log") || die "Can't open for writing: $!";		
	
	# Process each line in the file.
	LINE: while (<ORIGINAL>) 
	{
		# Inidicate that you are in the section of the file containing
		# the help nodes and not the header area.
		if (/<topic/)
		{
			$in_node_section = 1;
		}	

		# Don't write any lines that are blank.
		if (/^\s*\n/ && $in_node_section == 1)
		{
			next LINE;
		}

		# Get the indentations levels of the current and preceding 
		# TOC headings. This information is embedded in the HXT file
		# by webworks and appears directly above opening node tags.
		# This information is not written to the adjusted HXT file.
		if(/previous_level\=(\d) current_level\=(\d)/)
		{
			$previous_level = $1;
			$current_level = $2;
			# Increment the count of currently open nodes.
			$open_node_count += 1;

			if ($current_level <= 3)
			{
				# Check if the previous node was one of ChapterTitleTOC, 
				# Section1TOC, or Section2TOC and set the section_increment
				# accordingly.
				if ($previous_level <= 3)
				{ 
					$section_increment = 0;
				}
				elsif ($in_sec2 == 1)
				{
        			$section_increment = 0;
    			}
    			elsif ($in_sec1 == 1)
    			{
        			$section_increment = 1;
    			}
    			else
    			{
        			$section_increment = 2;
    			}	    				
			}
			else 
			{
				$section_increment = 0;
			}

			$level_difference = $previous_level - ($current_level + $section_increment);
#@@@ What about a h2 following a sec1 or sec2?			
			# Print an error message to the logfile if a heading does not follow
			# the style guide; eg. a h3 follows a h1, should be h2 following a h1.
			# Note: The rest of the log info is written out later in this script.
			if (($previous_level > 2 && $level_difference < -1) ||
				($previous_level <= 3 && $current_level >= 5 ))
			{
				print LOGFILE "Error: Heading does not follow style guide\n" || die "Can't open for writing: $!";
				print "There is a heading error in the TOC.  See the logfile for details\n";
				$heading_error = 1;
			}

			# Once the $level_difference has been established, set how much to 
			# increment subsequent toc entries based on whether you are in a 
			# Section1TOC or a Section2TOC or ChapterTitleTOC node.
			# $current_level = 1 means we are processing a ChapterTitleTOC node.
			if ($current_level == 1)
			{
				$in_sec1 = 0;
				$in_sec2 = 0;
			}
			# $current_level = 2 means we are processing a Section1TOC node.
			elsif ($current_level == 2)		
			{
				$in_sec1 = 1;
				$in_sec2 = 0;
			}
			# $current_level = 3 means we are processing a Section2TOC node.
			elsif ($current_level == 3)
			{
				$in_sec1 = 1;
				$in_sec2 = 1;
			}
   			# Don't write the current line to the file, it is not used
  			# by the Microsoft help compiler.
			next LINE;
		}
		
		# Variables used to left pad the string being written back
		# to the HxT file.
		(my $padding) = "  ";
		(my $node_string) = "";
		

		# Write out closing nodes equal in number to the difference
		# between the preceeding and current heading levels.
		while ($level_difference >= 0 && $open_node_count > 0)
		{
			$node_string = $padding x ($open_node_count - 1);
	    	$node_string = $node_string."<\/topic>\n";
		    print COPY $node_string || die "Can't write to file: $!";
		    $output_line_num += 1;
			$level_difference -= 1;
			$open_node_count -= 1;
	    }


	    # Check for the end of the TOC file and write out the remaining 
	    # close node tags so that all of the open nodes are now closed.
		if (not /<\/topic>/)
		{
			# Check to see if there was a heading error
			if ($heading_error == 1)
			{
				# Grab the title from the node and print it to the logfile
				/(Title = \".*\")/;
				print LOGFILE "    Output file, line ".$output_line_num.": ".$1."\n" || die "Can't write to file: $!";
				$heading_error = 0;
			}
			
	  	    # Write out the current opening node with padding appropriate 
	  	    # to its heading level.
			$node_string = $padding x ($open_node_count);
	    	$node_string = $node_string.$_;
			print COPY $node_string || die "Can't write to file: $!";
			$output_line_num += 1;
		}
		else
		{
			# Write out the remaining help node closing tags.
			while ($open_node_count > 0)
			{
				$node_string = $padding x ($open_node_count);
	    		$node_string = $node_string."<\/topic>\n";
			    print COPY $node_string || die "Can't write to file: $!";
			    $output_line_num += 1;
    			$open_node_count-= 1;
		    }
		    # Write last line to the file.
		    print COPY $_ || die "Can't write to file: $!";
		    $output_line_num += 1;
	    }
	    
	}
	
	close (ORIGINAL);
    close (COPY);
   	close (LOGFILE);	
}	


### # # #  #  #   #   #    #     #      #      #     #    #   #  #  # # # ###
#
# sub make_writable
#
# Parameters: $file_to_process contains the name of the file to make 
#	      writeable.
# 
# Get rid of the READONLY bit
# 
	
sub make_writable 
{
	# Make sure file is writable
	my $attribs = 1; # Silence the warning
	die "Can't find $_[0]: $!" unless -e $_[0];
	Win32::File::GetAttributes($_[0], $attribs) || die "Can't get file attributes: $!";

	if($attribs & READONLY) 
	{
		# Subtracting 1 seems the clearest way to modify only the READONLY bit
		$attribs = $attribs - 1;
		Win32::File::SetAttributes($_, $attribs) || die "Can't set file attributes: $!";
	}
}